START TRANSACTION ;

-- --------------------------------------------------------
# Добавить колонку Degree в зачетки для упрощения кода миграции
ALTER TABLE `record_books`
  ADD COLUMN `Degree` VARCHAR(10),
  ADD KEY (`Degree`);

-- --------------------------------------------------------
# generate fake record books
# Заполнить таблицу record_books фейковыми значениями следующего вида:
# fake-<degree>-<studentId>
# <degree> - уровень подготовки (b - бакалавр, m - магистр)
# <studentId> - идентификатор студента
DELIMITER //
CREATE PROCEDURE RecordBooks_Fill()
BEGIN
  DECLARE done INT DEFAULT FALSE;
  DECLARE vStudentID INT;
  DECLARE vDegree VARCHAR(10);
  DECLARE vRecordBookNum VARCHAR(50);
  DECLARE cur CURSOR FOR
    SELECT view_students.StudentID AS StudentID, view_students.Degree AS Degree FROM `view_students`
    GROUP BY view_students.Degree, view_students.StudentID;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cur;
  read_loop: LOOP
    FETCH cur INTO vStudentID, vDegree;
    IF done THEN
      LEAVE read_loop;
    END IF;
    SET vRecordBookNum = REPLACE(REPLACE("fake-<degree>-<studentid>", "<degree>", vDegree),
                                                                 "<studentid>", vStudentID);
    INSERT `record_books` (`StudentID`, `ExternalID`, `PlanID`, `Degree`)
      VALUES (vStudentID, vRecordBookNum, 0, vDegree);
  END LOOP;
  CLOSE cur;
END //

DELIMITER ;

CALL RecordBooks_Fill();
DROP PROCEDURE RecordBooks_Fill;

COMMIT ;

START TRANSACTION;

-- --------------------------------------------------------
# groups_students
# 1. создать поле RecordBookID
# 2. заполнить поле RecordBookID
# Удаление старых связей и создание новых после всех использований этой таблицы

ALTER TABLE `students_groups`
  ADD COLUMN `RecordBookID` int(11),
  ADD KEY  (`RecordBookID`);

COMMIT;

START TRANSACTION;
UPDATE `students_groups`
  INNER JOIN study_groups on study_groups.ID = students_groups.GroupID
  INNER JOIN grades on grades.ID = study_groups.GradeID
  INNER JOIN record_books on
                            record_books.Degree = grades.Degree and
                            record_books.StudentID = students_groups.StudentID
SET students_groups.RecordBookID = record_books.ID;

COMMIT;
#
# -- --------------------------------------------------------
# # logs_bind_students
# # 1. создать поле RecordBookID и связи для него
# # 2. заполнить поле RecordBookID
# # 3. удалить все связи поля StudentID и потом само поле
#
# ALTER TABLE `logs_binds_students`
#   ADD COLUMN `RecordBookID` int(11),
#   DROP INDEX `StudentID`,
#   ADD CONSTRAINT `logs_binds_students_ibfk_3` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`);
#
# UPDATE logs_binds_students AS target,
#   (SELECT MIN(view_recordbook_for_submodule.StudentID) AS StudentID,
#           MIN(view_recordbook_for_submodule.RecordBookID) AS RecordBookID,
#      view_recordbook_for_submodule.DisciplineID
#    FROM view_recordbook_for_submodule
#    GROUP BY view_recordbook_for_submodule.DisciplineID) AS matching
# SET target.RecordBookID = matching.RecordBookID WHERE target.StudentID = matching.StudentID
#                                                       AND target.DisciplineID = matching.DisciplineID;
#
# ALTER TABLE `logs_binds_students`
#   DROP COLUMN `StudentID`;
#
# -- --------------------------------------------------------
# # groups_students
# # 1. создать поле RecordBookID и связи для него
# # 2. заполнить поле RecordBookID
# # 3. удалить все связи поля StudentID и потом само поле
#
# ALTER TABLE `students_groups`
#   ADD COLUMN `RecordBookID` int(11),
#   DROP INDEX `StudentID`,
#   DROP INDEX `StudentID_2`,
#   ADD CONSTRAINT `students_groups_ibfk_1` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`),
#   ADD UNIQUE `SemesterID_RecordBookID` (`SemesterID`, `RecordBookID`);
#
#
# UPDATE students_groups AS target,
#   (SELECT DISTINCT students_groups.StudentID AS StudentID,
#                     record_books.ID AS RecordBookID,
#                     groups.ID AS GroupID
#   FROM `students_groups`
#     INNER JOIN groups ON groups.ID = students_groups.GroupID,
#     INNER JOIN record_books ON record_books.StudentID = students_groups.StudentID
#                                AND record_books.Degree = groups.Degree) AS matching
# SET target.RecordBookID = matching.RecordBookID WHERE target.StudentID = matching.StudentID
#                                                       AND target.GroupID = matching.GroupID;
#
# ALTER TABLE `students_groups`
#   DROP COLUMN `StudentID`;
#
# # Удалить временную колонку Degree в зачетках
# ALTER TABLE `record_books`
#   DROP COLUMN `Degree`;
#
# # Удалить представление
# DROP VIEW view_recordbook_for_submodule;

